package com.example.survey.enumeration;

import java.util.HashSet;
import java.util.Set;

/**
 * @author Pope
 */

public enum AuthEnum {

    /**
     * 管理员权限，可以访问所有接口
     */
    ADMIN(0, "管理员", new HashSet<String>() {{
        add("/investigator/investigator : POST");
        add("/investigationRecord/underReviewRecord : GET");
        add("/investigationRecord/underReviewRecordCount : GET");
        add("/investigationRecord/investigationRecord : POST");
        add("/investigationRecord/investigationRecord : PUT");
        add("/investigationRecord/underReviewRecord : PUT");
        add("/respondent/respondent : POST");
        add("/respondent/respondent : GET");
        add("/user/userRole : PUT");
        add("/role/role : POST");

    }}),

    /**
     * 查询调查记录的权限
     */
    RETRIEVE_RECORD(1, "查询调查记录的权限", new HashSet<String>() {{
        add("/investigationRecord/underReviewRecord : GET");
        add("/investigationRecord/underReviewRecordCount : GET");
    }}),

    /**
     * 提交记录的权限
     */
    CREATE_RECORD(2, "提交调查记录权限", new HashSet<String>() {{
        add("/investigationRecord/investigationRecord : POST");
    }}),

    /**
     * 修改调查记录信息的权限
     */
    UPDATE_RECORD(3, "修改调查记录权限", new HashSet<String>() {{
        add("/investigationRecord/investigationRecord : PUT");
    }}),

    /**
     * 审核调查记录权限
     */
    REVIEW_RECORD(4, "审核调查记录权限", new HashSet<String>() {{
        add("/investigationRecord/underReviewRecord : PUT");
    }}),

    /**
     * 删除调查记录权限
     */
    DELETE_RECORD(5, "删除调查记录权限", new HashSet<String>() {{

    }}),

    /**
     * 分析调查记录的权限
     */
    ANALYSE_RECORD(6, "分析调查记录权限", new HashSet<String>() {{

    }}),

    /**
     * 可以增删改流调人员信息
     */
    CUD_INVESTIGATOR(7, "增删改流调人员信息的权限", new HashSet<String>() {{

    }}),

    /**
     * 查询流调人员信息
     */
    RETRIEVE_INVESTIGATOR(8, "查询流调人员信息的权限", new HashSet<String>() {{

    }}),

    /**
     * 增删改调查对象信息的权限
     */
    CUD_RESPONDENT(9, "增删改调查对象信息的权限", new HashSet<String>() {{
        add("/respondent/respondent : POST");
    }}),

    /**
     * 查询调查对象信息的权限
     */
    RETRIEVE_RESPONDENT(10, "查询调查对象信息的权限", new HashSet<String>() {{
        add("/respondent/respondent : GET");
    }});


    private int code;
    private String name;
    private Set<String> routes;

    AuthEnum(int code, String name, Set<String> routes) {
        this.code = code;
        this.name = name;
        this.routes = routes;
    }

    public static AuthEnum getRoleEnum(int code) {
        for (AuthEnum value : AuthEnum.values()) {
            if (value.code == code) {
                return value;
            }
        }
        return null;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Set<String> getRoutes() {
        return routes;
    }

    public void setRoutes(Set<String> routes) {
        this.routes = routes;
    }
}
