package com.example.survey.service.impl;

import com.example.survey.dao.InvestigationRecordDao;
import com.example.survey.dao.InvestigatorDao;
import com.example.survey.dao.RespondentDao;
import com.example.survey.entity.InvestigationRecord;
import com.example.survey.entity.inner.OperationInformation;
import com.example.survey.service.InvestigationRecordService;
import com.example.survey.vo.InvestigationRecordVo;
import com.example.survey.vo.ReviewVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author Pope
 */
@Service
public class InvestigationRecordServiceImpl implements InvestigationRecordService {

    @Autowired
    private InvestigationRecordDao investigationRecordDao;

    @Autowired
    private InvestigatorDao investigatorDao;

    @Autowired
    private RespondentDao respondentDao;

    @Override
    public boolean addInvestigationRecord(InvestigationRecordVo vo) {
        InvestigationRecord record = InvestigationRecord.getInvestigationRecord(vo);
        //生成版本信息
        record.setState(InvestigationRecord.UNDER_REVIEW);
        record.setVersion(vo.getInvestigatorPhone() + new Date().toString());
        List<OperationInformation> opList = new ArrayList<>();
        OperationInformation op = new OperationInformation();
        op.setType(OperationInformation.SUBMIT);
        op.setTime(new Date());
        op.setInvestigatorPhone(vo.getInvestigatorPhone());
        op.setVersion(vo.getInvestigatorPhone() + new Date().toString());
        op.setMsg(vo.getMsg());
        op.setResult("提交成功");
        opList.add(op);
        record.setOperationInformationList(opList);


        //如果不存在对应id的调查对象
        if (!respondentDao.existRespondent(record.getIdNumber())) {
            //TODO 改为抛出相应异常
            return false;
        }

        //如果已经存在则失败
        if (investigationRecordDao.existInvestigationRecord(record.getIdNumber(), InvestigationRecord.UNDER_REVIEW, InvestigationRecord.REVIEWED)) {
            return false;
        }

        //如果不存在对应id的流调人员
        if (!investigatorDao.existInvestigator(record.getInvestigatorPhone())) {
            //TODO 改为抛出相应异常
            return false;
        }
        //TODO  插入消息队列


        return investigationRecordDao.insertInvestigationRecord(record);
    }

    @Override
    public List<InvestigationRecord> listUnderReviewRecordPageByInvestigationPhone(String investigatorPhone, int currentPage, int pageSize, String state, String idNumber, String version, String questionnaireNumber, Boolean diseased) {
        //TODO 从消息队列进行筛选

        return investigationRecordDao.listLimitInvestigationRecord(investigatorPhone, pageSize * currentPage, pageSize, state, idNumber, version, questionnaireNumber, diseased);
    }


    @Override
    public long countUnderReviewRecordByInvestigatorPhone(String investigatorPhone) {
        //TODO 从消息队列获取调查记录数量

        return investigationRecordDao.countInvestigationRecordByInvestigatorPhone(investigatorPhone, InvestigationRecord.UNDER_REVIEW);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void changeInvestigationRecord(InvestigationRecordVo vo) {
        //创建新记录
        InvestigationRecord oldRecord = investigationRecordDao.selectInvestigationRecord(vo.getIdNumber(), InvestigationRecord.REVIEWED);
        if(oldRecord == null){
            oldRecord = investigationRecordDao.selectInvestigationRecord(vo.getIdNumber(), InvestigationRecord.NOT_PASS);
        }
        InvestigationRecord newRecord = InvestigationRecord.getInvestigationRecord(vo);

        //生成版本信息
        newRecord.setState(InvestigationRecord.UNDER_REVIEW);
        newRecord.setVersion(vo.getInvestigatorPhone() + new Date().toString());
        //生成操作记录
        List<OperationInformation> opList = oldRecord.getOperationInformationList();
        OperationInformation op = new OperationInformation();
        op.setType(OperationInformation.CHANGE);
        op.setTime(new Date());
        op.setInvestigatorPhone(vo.getInvestigatorPhone());
        op.setVersion(vo.getInvestigatorPhone() + new Date().toString());
        op.setMsg(vo.getMsg());
        op.setResult("提交成功");
        opList.add(op);
        newRecord.setOperationInformationList(opList);

        //存入新纪录
        //原纪录标记为已归档
        investigationRecordDao.updateRecordState(oldRecord.getIdNumber(), InvestigationRecord.REVIEWED, InvestigationRecord.OUT_OF_DATE);
        investigationRecordDao.insertInvestigationRecord(newRecord);

        //TODO 将修改请求加入消息队列

    }

    @Override
    public boolean reviewRecord(ReviewVo vo) {

        //如果调查记录表不存在
        if (!investigationRecordDao.existInvestigationRecord(vo.getIdNumber(), InvestigationRecord.UNDER_REVIEW)) {
            //TODO 抛出相应异常
            return false;
        }
        //如果审核人不存在
        if (!investigatorDao.existInvestigator(vo.getReviewerPhone())) {
            //TODO 抛出相应异常
            return false;
        }

        InvestigationRecord record = investigationRecordDao.selectInvestigationRecord(vo.getIdNumber(), InvestigationRecord.UNDER_REVIEW);

        //审核结果与版本信息
        record.setState(vo.isPass() ? InvestigationRecord.REVIEWED : InvestigationRecord.NOT_PASS);
        record.setVersion(vo.getReviewerPhone() + new Date().toString());

        //生成操作记录
        List<OperationInformation> opList = record.getOperationInformationList();
        OperationInformation op = new OperationInformation();
        op.setType(OperationInformation.REVIEW);
        op.setTime(new Date());
        op.setInvestigatorPhone(vo.getReviewerPhone());
        op.setVersion(vo.getReviewerPhone() + new Date().toString());
        op.setMsg(vo.getMsg());
        op.setResult(vo.isPass() ? "审核通过" : "审核不通过");
        opList.add(op);
        record.setOperationInformationList(opList);

        //更新数据库记录
        investigationRecordDao.reviewRecord(record);

        return true;
    }
}
