package com.example.survey.service.impl;

import com.example.survey.dao.InvestigatorDao;
import com.example.survey.dao.RoleDao;
import com.example.survey.dao.UserDao;
import com.example.survey.dto.LoginDto;
import com.example.survey.entity.Investigator;
import com.example.survey.entity.Role;
import com.example.survey.entity.User;
import com.example.survey.enumeration.AuthEnum;
import com.example.survey.service.UserService;
import com.example.survey.util.TokenUtil;
import com.example.survey.vo.LoginVo;
import com.example.survey.vo.SignupVo;
import com.example.survey.vo.UserRoleVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author Pope
 */
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserDao userDao;
    @Autowired
    private TokenUtil tokenUtil;
    @Autowired
    private RoleDao roleDao;
    @Autowired
    private InvestigatorDao investigatorDao;

    @Override
    public LoginDto matchAuth(LoginVo loginVo) {
        User user = userDao.selectUser(loginVo.getPhoneNumber(), loginVo.getPassword());
        if (user == null) {
            return null;
        }
        LoginDto loginDto = new LoginDto();

        //判断是否已经登录过
        if (tokenUtil.existToken(user.getPhoneNumber())) {
            String oldToken = (String) tokenUtil.getValue(user.getPhoneNumber());
            if (tokenUtil.existToken(oldToken)) {
                //已经登录，将旧token过期，
                tokenUtil.expire(oldToken);
            }
        }

        //生成新的token并存入redis
        String newToken = UUID.randomUUID().toString();
        tokenUtil.setPhoneNumberAndToken(user.getPhoneNumber(), newToken);

        //生成角色列表
        Set<String> roleNameSet = new HashSet<>();
        for (Role role : user.getRoleList()) {
            Set<String> routeSet = new HashSet<>();
            for (AuthEnum authEnum : role.getAuthoritySet()) {
                routeSet.addAll(authEnum.getRoutes());
            }
            tokenUtil.setAuthAndRoute(role.getName(), routeSet);
            roleNameSet.add(role.getName());
        }
        tokenUtil.setTokenAndAuth(newToken, roleNameSet);

        loginDto.setToken(newToken);
        loginDto.setRoleList(user.getRoleList());
        loginDto.setDepartmentList(user.getDepartmentList());

        return loginDto;
    }

    @Override
    public boolean addUser(SignupVo signupVo) {
        User user = new User(signupVo);

        //初始化为流调人员权限
        Role investigatorRole = roleDao.selectRole("流调人员");
        List<Role> roleList = user.getRoleList();
        roleList.add(investigatorRole);
        user.setRoleList(roleList);
        Investigator investigator = new Investigator();
        investigator.setName(user.getUsername());
        investigator.setIdNumber(user.getIdNumber());
        investigator.setPhoneNumber(user.getPhoneNumber());
        return userDao.insertUser(user) && investigatorDao.insertInvestigator(investigator);
    }

    @Override
    public void modifyRoles(UserRoleVo userRoleVo) {
        List<Role> roleList = new ArrayList<>();
        for (String roleName : userRoleVo.getRoleNameList()) {
            roleList.add(roleDao.selectRole(roleName));
        }

        roleDao.updateUserRoles(userRoleVo.getPhoneNumber(), roleList);
    }
}
